package com.mit.jna.utils.broadcast;

import com.alibaba.fastjson.annotation.JSONField;
import com.mit.jna.configuration.YamlPropertySourceFactory;
import com.mit.jna.dto.MulticastDTO;
import lombok.Data;
import lombok.experimental.Accessors;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.function.Consumer;

@Data
@Component
@PropertySource(value = {"classpath:broadcast.yml"}, factory = YamlPropertySourceFactory.class)
@ConfigurationProperties(prefix = "broadcast-device")
@Slf4j
public class BroadcastDevice {

    private String nicIP;
    private String multicastIP;
    private Integer nodeID;
    private Integer audioStreamFormat;
    private List<Multicast> multicasts;

    @JSONField(serialize = false)
    private Map<String, Multicast> mapMulticast;

    @PostConstruct
    private void transfer() {
        // map-multicast
        mapMulticast = new HashMap<>();
        for (Multicast multicast : multicasts) {
            if (multicast.getEnable())
                mapMulticast.put(multicast.getId(), multicast);
        }
    }

    public String getNPMSId(MulticastDTO multicastDTO) {
        Multicast multicast = mapMulticast.get(multicastDTO.getMulticastId());
        Optional<Device> npms = multicast.getDevices().stream().filter(c -> c.getType().equals("2")).findFirst();
        Device device = npms.orElseGet(() -> new Device().setId(""));
        return device.getId();
    }

    public String getZone2String(MulticastDTO multicastDTO) {
        Multicast multicast = mapMulticast.get(multicastDTO.getMulticastId());
        Optional<Device> dcs = multicast.getDevices().stream().filter(c -> c.getType().equals("1")).findFirst();
        if (dcs.isPresent()) {
            return String.format("%4s%2s", Integer.toHexString(Integer.parseInt(dcs.get().getId())), zone2Hex(multicastDTO.getZoneIdBin())).replace(' ', '0');
        }
        return "";
    }

    public void addDevices(Consumer<List<String>> addDevice) {
        this.getMulticasts().stream().forEach(c -> {
            c.getDevices().stream().forEach(d -> {
                List<String> deviceArgs = Arrays.asList(d.getId(), d.getType(), d.getIp());
                addDevice.accept(deviceArgs);
            });
        });
    }

    // 将zone（二进制）转换为16进制
    public String zone2Hex(String zoneId) {
        Integer zoneId01 = Integer.valueOf(zoneId, 2);
        return Integer.toHexString(zoneId01);
    }

    // 监听域
    public List<String> monitorZones(MulticastDTO multicastDTO) {
        Multicast multicast = mapMulticast.get(multicastDTO.getMulticastId());
        Optional<Device> dcsOptional = multicast.getDevices().stream().filter(c -> c.getType().equals("1")).findFirst();
        String dcs = dcsOptional.get().getId();
        List<String> dcsZones = new ArrayList<>();
        for (Integer zone : multicastDTO.getZones()) {
            String key = String.format("%s-%d", dcs,zone);
            dcsZones.add(key);
        }
        return dcsZones;
    }
}

// 广播
@Data
class Multicast {
    private String id;
    private Boolean enable;
    private List<Device> devices;
    private List<Zone> zones;
}

@Data
class Zone {
    private String id;
    private Boolean enabled;
}

@Data
@Accessors(chain = true)
class Device {
    private String id;
    private String type;
    private String ip;
    private String desc;
}