package com.mit.jna.utils.broadcast;

import com.mit.jna.dto.MulticastDTO;
import com.mit.jna.listener.HoneywellListener;
import com.mit.jna.utils.broadcast.sdk.HoneywellLibrary;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.sound.sampled.*;
import java.util.List;
import java.util.Random;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Component
@Slf4j
public class HoneywellPlayer implements Player {
    @Autowired
    BroadcastDevice broadcastDevice;

    @Override
    public Boolean openVoice(MulticastDTO multicastDTO) {
        String npmsId = broadcastDevice.getNPMSId(multicastDTO);
        String zoneString = broadcastDevice.getZone2String(multicastDTO);
        int i = HoneywellLibrary.INSTANCE.X618_StartNPMSTask(Integer.parseInt(npmsId), 3, 100, zoneString);
        return i == 0 ? true : false;
    }

    @Override
    public Boolean closeVoice(MulticastDTO multicastDTO) {
        String npmsId = broadcastDevice.getNPMSId(multicastDTO);
        int i = HoneywellLibrary.INSTANCE.X618_StopNPMSTask(Integer.parseInt(npmsId), 3);
        return i == 0;
    }

    @Override
    public void playAudio(MulticastDTO multicastDTO) {

        String filePaths = multicastDTO.getFilePaths().stream().collect(Collectors.joining("|"));

        // 设置监听域
        List<String> dcsZones = broadcastDevice.monitorZones(multicastDTO);
        for (String dcsZone : dcsZones) {
            PlayAudio playAudio = new PlayAudio(filePaths);
            HoneywellListener.monitorZoneAudio.put(dcsZone, playAudio);
        }

        // 播放音频
        String zoneString = broadcastDevice.getZone2String(multicastDTO);
        int taskId = new Random().nextInt(5000) + 1;
        HoneywellLibrary.INSTANCE.X618_StartAudioFileTask(
                taskId,                     // 任务ID
                100,                // 音量100%
                70,                 // 优先级70
                0,                  // 不中断恢复
                zoneString,                 // DCS的ID, ZONE的分区1~8
                0,         // 不打开输出干接点
                filePaths,                  // 文件路径
                multicastDTO.getRepeat()    // 重复次数
        );
    }

    @Override
    public void monitorZoneAudio(String dcsId, String zoneId) {
        String key = String.format("%s-%s", dcsId, zoneId);
        PlayAudio playAudio = HoneywellListener.monitorZoneAudio.get(key);
        if (playAudio != null) {
            Double remaining = playAudio.remainingTime();
            if (remaining > 0) {
                // 还有可播放的时间，可进行监听
                playAudio.playRemainingAudio();
            }
        }
    }
}
