package com.mit.jna;

import com.alibaba.fastjson.JSONObject;
import com.esotericsoftware.yamlbeans.YamlReader;
import com.mit.jna.utils.broadcast.BroadcastDevice;
import com.mit.jna.utils.broadcast.BroadcastFile;
import org.assertj.core.error.ShouldBeEqualByComparingFieldByFieldRecursively;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.io.ClassPathResource;

import javax.sound.sampled.*;
import java.applet.Applet;
import java.applet.AudioClip;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.TimeUnit;

@SpringBootTest
class JnaTestApplicationTests {

    @Autowired
    BroadcastDevice broadcastDevice;

    @Autowired
    BroadcastFile broadcastFiles;

    @Test
    void contextLoads() throws IOException {
        ClassPathResource classPathResource = new ClassPathResource("broadcast.yml");
        InputStream inputStream = classPathResource.getInputStream();
        Reader reader = new InputStreamReader(inputStream, "UTF-8");
        YamlReader ymlReader = new YamlReader(reader);
        Map obj = (Map) ymlReader.read();
        Object o = obj.get("broadcast-device");
        String s = JSONObject.toJSONString(o);
        BroadcastDevice device = JSONObject.parseObject(s, BroadcastDevice.class);
        System.out.println(device.getNicIP());
    }

    @Test
    void playAudio() throws MalformedURLException, InterruptedException {
        File file = new File("E:\\project\\北京apm\\项目\\APM\\X618 SDK V1.0.2（2020-12-4）\\X618 SDK V1.0.2\\examples\\test441.wav");
        //创建audioclip对象
        AudioClip audioClip = null;
        //将file转换为url
        URL url = file.toURI().toURL();
        audioClip = Applet.newAudioClip(file.toURI().toURL());
        audioClip.play();
        //循环播放	播放一次可以使用audioClip.play
        Thread.sleep(5000);
    }

    @Test
    void playPCM() throws LineUnavailableException, IOException {
        // select audio format parameters
        AudioFormat af = new AudioFormat(44100, 16, 1, true, false);
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, af);
        SourceDataLine line = (SourceDataLine) AudioSystem.getLine(info);

        // 准备数据
        byte[] buffer = new byte[4096];
        ClassPathResource classPathResource = new ClassPathResource("pcm.pcm");
        InputStream inputStream = classPathResource.getInputStream();

        // prepare audio output
        line.open(af, 4096);
        line.start();

        while (inputStream.read(buffer) > 0) {
            line.write(buffer, 0, buffer.length);
        }

        // 关闭
        line.drain();
        line.stop();
        line.close();
    }

    // volatile ArrayBlockingQueue<byte[]> audioBytes = new ArrayBlockingQueue<>(100);

    @Test
    void audioBlockQueue() throws InterruptedException {

        ArrayBlockingQueue<byte[]> audioBytes = new ArrayBlockingQueue<>(100);

        // 写入音频流
        new Thread(() -> {
            System.out.println("写入音频流");
            ClassPathResource classPathResource = new ClassPathResource("pcm.pcm");
            try {
                InputStream inputStream = classPathResource.getInputStream();
                byte[] buffer = new byte[4096];
                while (inputStream.read(buffer) > 0) {
                    byte[] copyBuffer = Arrays.copyOf(buffer, buffer.length);
                    audioBytes.put(copyBuffer);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }, "a").start();

        Thread.sleep(300);

        // select audio format parameters
        AudioFormat af = new AudioFormat(44100, 16, 1, true, false);
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, af);
        SourceDataLine line = null;
        try {
            line = (SourceDataLine) AudioSystem.getLine(info);
        } catch (LineUnavailableException e) {
            e.printStackTrace();
        }

        // prepare audio output
        try {
            line.open(af, 4096);
            line.start();
        } catch (LineUnavailableException e) {
            e.printStackTrace();
        }

        do {
            byte[] buffer = audioBytes.poll(1, TimeUnit.SECONDS);
            line.write(buffer, 0, buffer.length);
        }
        while (audioBytes.peek() != null);

        if (line != null) {
            // 关闭
            line.drain();
            line.stop();
            line.close();
        }
    }

    @Test
    void audioMonitor() {

        // 1s/4kb，演练10s后监听，也就是40kb的数据

        // select audio format parameters
        AudioFormat af = new AudioFormat(44100, 16, 1, true, false);
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, af);
        SourceDataLine line = null;
        try {
            line = (SourceDataLine) AudioSystem.getLine(info);
        } catch (LineUnavailableException e) {
            e.printStackTrace();
        }

        // prepare audio output
        try {
            line.open(af, 4096);
            line.start();
        } catch (LineUnavailableException e) {
            e.printStackTrace();
        }

        int i = 0;
        //ClassPathResource classPathResource = new ClassPathResource("pcm.pcm");
        ClassPathResource classPathResource = new ClassPathResource("test441.wav");
        try {
            InputStream inputStream = classPathResource.getInputStream();
            byte[] buffer = new byte[4096];
            while (inputStream.read(buffer) > 0) {
                if (i >= 10) {
                    line.write(buffer, 0, buffer.length);
                }
                i++;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        if (line != null) {
            // 关闭
            line.drain();
            line.stop();
            line.close();
        }

    }
}
